Package: recoup
Type: Package
Title: An R package for the creation of complex genomic profile plots
Author: Panagiotis Moulos <moulos@fleming.gr>
Maintainer: Panagiotis Moulos <moulos@fleming.gr>
Depends: R (>= 4.0.0), GenomicRanges, GenomicAlignments, ggplot2,
        ComplexHeatmap
Imports: BiocGenerics, biomaRt, Biostrings, circlize, Seqinfo,
        GenomicFeatures, graphics, grDevices, httr, IRanges, methods,
        parallel, RSQLite, Rsamtools, rtracklayer, S4Vectors, stats,
        stringr, txdbmaker, utils
Suggests: GenomeInfoDb, grid, BiocStyle, knitr, rmarkdown, zoo, RUnit,
        BiocManager, BSgenome, RMySQL
Description: recoup calculates and plots signal profiles
    created from short sequence reads derived from Next
    Generation Sequencing technologies. The profiles
    provided are either sumarized curve profiles or heatmap
    profiles. Currently, recoup supports genomic profile
    plots for reads derived from ChIP-Seq and RNA-Seq
    experiments. The package uses ggplot2 and ComplexHeatmap
    graphics facilities for curve and heatmap coverage
    profiles respectively.
License: GPL (>= 3)
Encoding: UTF-8
LazyLoad: yes
LazyData: yes
URL: https://github.com/pmoulos/recoup
biocViews: ImmunoOncology, Software, GeneExpression, Preprocessing,
        QualityControl, RNASeq, ChIPSeq, Sequencing, Coverage, ATACSeq,
        ChipOnChip, Alignment, DataImport
VignetteBuilder: knitr
Authors@R: c(person(given="Panagiotis", family="Moulos",
    email="moulos@fleming.gr", role=c("aut", "cre")))
Version: 1.40.0
Date: 2026-03-23
Collate: 'annotation.R' 'argcheck.R' 'coverage.R' 'count.R' 'plot.R'
        'profile.R' 'query.R' 'ranges.R' 'recoup.R' 'util.R'
git_url: https://git.bioconductor.org/packages/recoup
git_branch: RELEASE_3_23
git_last_commit: 4a9a71f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:09:41 UTC; biocbuild
