import(biomaRt)
import(ComplexHeatmap)
import(GenomicAlignments)
import(GenomicRanges)
import(ggplot2)
import(parallel)
import(Rsamtools)
import(RSQLite)
import(utils)

importFrom("BiocGenerics",match)
importFrom("BiocGenerics",unlist)
importFrom("BiocGenerics",which)
importFrom("Biostrings",alphabetFrequency)
importFrom("biomaRt",useMart)
importFrom("biomaRt",getBM)
importFrom("circlize",colorRamp2)
importFrom("Seqinfo",genome)
importFrom("Seqinfo",isCircular)
importFrom("Seqinfo",seqinfo)
importFrom("Seqinfo",Seqinfo)
importFrom("Seqinfo",seqlevels)
importFrom("Seqinfo",seqlengths)
importFrom("Seqinfo","seqlengths<-")
#importFrom("GenomicFeatures",makeTxDbFromGFF)
importFrom("GenomicFeatures",exons)
importFrom("GenomicFeatures",genes)
importFrom("GenomicFeatures","seqlevels<-")
importFrom("GenomicFeatures",transcripts)
importFrom("GenomicFeatures",threeUTRsByTranscript)
importFrom("graphics",plot)
importFrom("grDevices",bmp)
importFrom("grDevices",col2rgb)
importFrom("grDevices",dev.new)
importFrom("grDevices",dev.off)
importFrom("grDevices",jpeg)
importFrom("grDevices",pdf)
importFrom("grDevices",png)
importFrom("grDevices",postscript)
importFrom("grDevices",tiff)
importFrom("httr",config)
importFrom("httr",set_config)
importFrom("IRanges",IRanges)
importFrom("IRanges",Views)
importFrom("IRanges",viewApply)
importFrom("methods",as)
importFrom("methods",is)
importFrom("rtracklayer",BigWigFile)
importFrom("rtracklayer",BigWigSelection)
importFrom("rtracklayer",import)
importFrom("rtracklayer",import.bed)
importFrom("rtracklayer",import.bw)
importFrom("rtracklayer",getSeq)
importFrom("S4Vectors",DataFrame)
importFrom("S4Vectors",match)
importFrom("S4Vectors",Rle)
importFrom("S4Vectors",runValue)
importFrom("S4Vectors",queryHits)
importFrom("S4Vectors",subjectHits)
importFrom("stats",aggregate)
importFrom("stats",approx)
importFrom("stats",cov)
importFrom("stats",kmeans)
importFrom("stats",lowess)
importFrom("stats",quantile)
importFrom("stats",smooth.spline)
importFrom("stats",spline)
importFrom("stats",var)
importFrom("stringr",str_match)
importFrom("txdbmaker",makeTxDbFromGFF)
importFrom("utils",download.file)
importFrom("utils",packageVersion)
importFrom("utils",read.delim)
importFrom("utils",unzip)

export(buildAnnotationDatabase)
export(buildCustomAnnotation)
export(calcCoverage)
export(coverageRef)
export(coverageRnaRef)
export(getAnnotation)
export(getBiotypes)
export(getInstalledAnnotations)
export(getr)
export(importCustomAnnotation)
export(kmeansDesign)
export(loadAnnotation)
export(mergeRuns)
export(preprocessRanges)
export(profileMatrix)
export(recoup)
export(recoupCorrelation)
export(recoupHeatmap)
export(recoupProfile)
export(recoupPlot)
export(removeData)
export(rpMatrix)
export(setr)
export(sliceObj)
